/*

Serial port Compact Flash data logger based for PIC 18F uCs
To be compiled with HI-TECH PIC18 Compilers
Freely distributable in this original form.
(c) 2006 Pasan Hettiarachchi
(c) Microsolutions
Any questions or comments?  admin@microsolutions.info

Derived from folllowing existing source codes
Compact Flash access: MintyMP3 project, http://www.ladyada.net/make/minty/ 
Copyright (c) 2004 Limor Fried <ladyada@media.mit.edu>

*/

#include<pic18.h>
#include "cf.h"
#include"delay.h"

#include"serial.h"
#include"always.h"

// Compact flash state and buffers. Should be defined in Main Program file
extern unsigned long CF_CurrSector; // the sector we're currently reading/writing
extern unsigned int CF_CurrByteInSector;  // what byte in that sector we're at
extern unsigned char CF_isValid; // CF seems ok
extern volatile unsigned char cfBuffer[512];



// write data to CF port addr
void CF_TaskFileWrite(unsigned char addr, unsigned char data){
	
   CF_A=addr;
   CF_WriteByte(data);
}


// read byte from CF port addr
unsigned char CF_TaskFileRead(unsigned char addr){
   CF_A=addr;
   return(CF_ReadByte());
}


/********************************************/
// write 1 unsigned char into given addr
/********************************************/
void CF_WriteByte(unsigned char data){
	
	waitforready();
	CF_DATA_DIR=OUTPUT_PORT;
   	CF_DATA_OUT=data;
   	asm("nop");
   	CF_WE=LOW;
   	asm("nop");
   	CF_WE=HIGH;
	asm("nop");
	CF_DATA_DIR=INPUT_PORT;
   	
}

/********************************************/
// read 1 unsigned char from given addr
/********************************************/

unsigned char CF_ReadByte(void){
	unsigned char data;
   	
   	waitforready();
   	CF_DATA_DIR=INPUT_PORT;
   	
   	asm("nop");
   	CF_OE=LOW;
	asm("nop");
   	data = CF_DATA_IN;
	asm("nop");
   	CF_OE=HIGH;
	asm("nop");
   	
	return data;
}

void CF_WriteInt(unsigned int data) {
   CF_WriteByte(data >> 8);
   CF_WriteByte(data & 0xff);
}

unsigned int CF_ReadInt(void) {
   unsigned char temp;
   unsigned int result;
   
   result = CF_ReadByte();
   temp = CF_ReadByte();
   result = (result<<8);
   result = (result|temp);
   return result;
}

void CF_WriteLong(unsigned long data) {
   CF_WriteByte(data >> 24);
   CF_WriteByte(data >> 16);
   CF_WriteByte(data >> 8);
   CF_WriteByte(data);
}

unsigned long CF_ReadLong(void) {
   unsigned int temp;
   unsigned long result;
   result = CF_ReadInt();
   temp = CF_ReadInt();
   
   result <<= 16;
   result |= temp;
   return result;
}

// skip bytes from current sector buffer
void CF_ReadSkipBytes(unsigned int count){
	
   //	printf("skipping %d bytes\n\r", count);
   CF_DATA_DIR=INPUT_PORT;
   while (count > 0) {
      waitforready();
      CF_OE=LOW;
      asm("nop");
      CF_OE=HIGH;
      count--;
   }
}

void CF_WriteSkipBytes(unsigned int count){
	
	CF_DATA_DIR=OUTPUT_PORT;
   	CF_DATA_OUT=0;
	
	while (count > 0) {
		waitforready();
   		CF_WE=LOW;
   		asm("nop");
   		CF_WE=HIGH;
		count--;
	}
	CF_DATA_DIR=INPUT_PORT;
   	
}

void CF_ReadSkipInt(unsigned char count){
   CF_ReadSkipBytes(count);
   CF_ReadSkipBytes(count);
}


void CF_StartSectorRead() {      // uses CF_CurrSector
   while (CF_isBusy() && (!CF_isReady())) ;

   //putst("Beginning read from current sector....\n");
   CF_TaskFileWrite(REG_SECTORCOUNT, 1);
   CF_TaskFileWrite(REG_SECTORNUM, CF_CurrSector);
   CF_TaskFileWrite(REG_CYLINDERLOW, CF_CurrSector >> 8);
   CF_TaskFileWrite(REG_CYLINDERHIGH, CF_CurrSector >> 16);
   CF_TaskFileWrite(REG_HEAD, 0xE0 | ((CF_CurrSector >> 24) & 0x0F));
   CF_TaskFileWrite(REG_COMMAND, COMMAND_READSECTOR);

   while ((CF_ReadByte()&0x88)!=0x08);      // check if valid??

   CF_A=REG_DATA;
}


void CF_StartSectorWrite() {      // uses CF_CurrSector
   while (CF_isBusy() && (!CF_isReady())) ;

   //putst("Beginning write to sector ");putchdec(CF_CurrSector);putch('\n');
   CF_TaskFileWrite(REG_SECTORCOUNT, 1);
   CF_TaskFileWrite(REG_SECTORNUM, CF_CurrSector);
   CF_TaskFileWrite(REG_CYLINDERLOW, CF_CurrSector >> 8);
   CF_TaskFileWrite(REG_CYLINDERHIGH,  CF_CurrSector >> 16);
   CF_TaskFileWrite(REG_HEAD, 0xE0 | (( CF_CurrSector >> 24) & 0x0F));
   CF_TaskFileWrite(REG_COMMAND, COMMAND_WRITESECTOR);
   DelayMs(1);
   while ((CF_ReadByte()&0x88)!=0x08) ;     // check if valid??

   CF_A=REG_DATA;
}



/***************************************/
// dump the contents of sectorbuff into a sector or vice versa
// assumes you've StartSectorWrite/Read 'd

void CF_ReadSectorIntoBuff() {
   unsigned int i;
   for (i=0; i < 512; i++) {
      cfBuffer[i] = CF_ReadByte();
   }
}

void CF_WriteBuffIntoSector() {
   unsigned int i;
   for (i=0; i < 512; i++) {
      CF_WriteByte(cfBuffer[i]);
      //putch('.');
   }
}


void CF_DumpSector() {      // uses CF_CurrSector
   unsigned int i;
   
   CF_StartSectorRead();

   CF_ReadSectorIntoBuff();
   for (i=0;; i++) {
	   
	   if ((cfBuffer[i]==0) |(i==512) ) break;
      putch(cfBuffer[i]);
  
   }
	putlf;	
}
	




unsigned char CF_isBusy(void) {
  return ((CF_TaskFileRead(REG_STATUS) & 0x80) == 0x80);
}

unsigned char CF_isReady(void) {
  return ((CF_TaskFileRead(REG_STATUS) & 0x40) == 0x40);
}


// check if the CD line is high
unsigned char is_CF_Out() {
   return CF_CD;             
}


// wait for the CF RDY line to go high, ie ready for next command
void waitforready(){
	while (!CF_READY);
	asm("nop");
}


void CF_ResetCard(void) {
 	CF_RESET=HIGH;
  	DelayMs(20);
  	CF_RESET=LOW;
	
  	DelayBigMs(400);
  	
}


void CF_Disable(void) {
   // default CF power down signals
   CF_OE=LOW;
   CF_WE=LOW;
   CF_CE=HIGH;
   CF_A=0;
   CF_RESET=HIGH;
   // presumption of invalidity
   CF_isValid=false;
}

unsigned char CF_Enable(void) {
   CF_Disable();
   CF_CE=LOW;
   putst("Resetting Card...\n");
   CF_ResetCard();
   if (is_CF_Out()) {
      CF_isValid = false;
      putst("No CF found!\n\r");
      return NO_CF;
   }
   CF_OE=HIGH;
   CF_WE=HIGH;
   CF_DATA_DIR=INPUT_PORT;
   CF_isValid = true;
   return CF_OK;
}



unsigned char CF_init(void){
	unsigned char result;
	putst("Waiting for CF card...\n\n");
	while(is_CF_Out());
	putst("Card found. Now initialising...\n");
	CF_Enable();
	CF_TaskFileWrite(REG_COMMAND,COMMAND_DIAGNOSTIC);
	result=CF_TaskFileRead(REG_ERROR);
	CF_A=REG_DATA;
	putst("CF Diagnostic result is.."); putchhex(result); putlf;
	
	return result;
}
	
