/*
 *	LCD interface example
 *	Uses routines from delay.c
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 or 8 bit mode
 *	
 */

#include	<pic18.h>
#include	"lcd.h"
#include	"delay.h"
 

/* send a command to the LCD */

void lcd_cmd(unsigned char c){
	LCD_WAIT; // may check LCD busy flag, or just delay a little, depending on lcd.h
	LCD_DATA_TRIS	 = OUTPUT_DATA;
	LCD_DATA = c;
	LCD_STROBE();
	LCD_DATA_TRIS	 = INPUT_DATA;
}

/* send data to the LCD */
void lcd_data(unsigned char c){
	LCD_WAIT; // may check LCD busy flag, or just delay a little, depending on lcd.h

	LCD_DATA_TRIS	 = OUTPUT_DATA;
	LCD_DATA = 0;
	LCD_RS = 1;
	
	LCD_DATA = c;
	LCD_STROBE();
	LCD_RS = 0;
	LCD_DATA_TRIS	 = INPUT_DATA;
}

/* write a string of chars to the LCD */

void lcd_puts(const char * s){
	while(*s)
		lcd_data(*s++);
}



void lcd_PutchDec(unsigned char c){

	//tens
	lcd_data((c/10)+'0');
	c=c%10;

	//ones
	lcd_data(c+'0');
}

void lcd_putchhex(unsigned char c){
	unsigned char temp;

	// transmits in hex

	temp=c;

	c=(c >> 4);
	if (c<10) c+=48; else c+=55;
	lcd_data(c);

	temp=(temp & 0x0F);
	if (temp<10) temp+=48; else temp+=55;
	lcd_data(temp);
}	

void lcd_putchlong(unsigned long c){
	unsigned int tempI;
	unsigned char tempC; 
	//hundred thousands
	lcd_data((c/100000)+'0');
	c=c%100000;
	//ten thousands
	lcd_data((c/10000)+'0');
	tempI=(unsigned int)(c%10000);
	//thousands
	lcd_data((tempI/1000)+'0');
	tempI=tempI%1000;
	//hindreds
	lcd_data((tempI/100)+'0');
	tempC=(unsigned char)(tempI%100);
	//tens
	lcd_data((tempC/10)+'0');
	tempC=tempC%10;

	//ones
	lcd_data(tempC+'0');
}

/* initialize the LCD */		
void lcd_init(){
	
	LCD_RS = 0;
	LCD_EN = 0;
	
	LCD_RS_TRIS	 = OUTPUT_PIN;
	LCD_EN_TRIS	 = OUTPUT_PIN;
	
	LCD_DATA_TRIS	 = OUTPUT_DATA;
	DelayMs(15);
	LCD_DATA	 = 0x3F;
	LCD_STROBE();
	DelayMs(5);
	LCD_DATA	 = 0x3F;
	LCD_STROBE();
	DelayUs(200);
	LCD_DATA	 = 0x3F;
	LCD_STROBE();
	
	lcd_cmd(0x38);

	lcd_cmd(0xC); //Display On, Cursor Off, 
	lcd_cmd(0x1); //Display Clear
	lcd_cmd(0x6); //Entry Mode
	lcd_cmd(0x80); //Initialize DDRAM address to zero
	
}



