/*
Serial port Compact Flash data logger based for PIC 18F uCs
To be compiled with HI-TECH PIC18 Compilers
Freely distributable in this original form.
(c) 2006 Pasan Hettiarachchi
(c) Microsolutions
Any questions or comments?  admin@microsolutions.info
*/
/*
Derived from folllowing freely redistributable existing source codes
Compact Flash access: MintyMP3 project, http://www.ladyada.net/make/minty/ 
Copyright (c) 2004 Limor Fried <ladyada@media.mit.edu>

Delay, Serial port and LCD routines: Shane Tolmie, www.microchipc.com 

*/
#include <pic18.h>
#include "rtc.h"
#include"serial.h"
#include"lcd.h"
#include"cf.h"
#include"delay.h"
#include "always.h"
#include "clk_freq.h"
#include <stdlib.h>



__CONFIG (1,UNPROTECT & ECDB4 & OSCSDIS);
__CONFIG (2,WDTDIS);
__CONFIG (4,LVPDIS & STVRDIS);
__CONFIG(5, UNPROTECT);


 void updateLCD();
 void findRecordsCount();
 void decodeTimeToAscii();
 void system_init();
 void setDateTime();
 void readRecords();
 unsigned char USARTGetString(char *buffer, unsigned char bufferLen);

 
 unsigned long recordsCount; //holds the already written records count
 unsigned long CF_CurrSector; // the sector we're currently reading/writing
 unsigned int CF_CurrByteInSector;  // what byte in that sector we're at
 unsigned char  CF_isValid; // CF seems ok
 volatile unsigned char cfBuffer[512];  //buffer used in writing and reading from CF card
 volatile bit newDatReceived;
 volatile dateTimeRTC dateTime ;		//the datime structure of type 'dateTimeRTC'
 volatile unsigned int recordLength;    // a variable to hold the lenght of the record received
 

 #define busyLED LATB7
 #define CTS_PIN LATC0
 #define DTR_PIN LATC0
 
 #define DEVICE_TYPE_DTE
 #define RECORDS_COUNT_SECTOR 100
	
void main(){
	
	unsigned int i;
	unsigned int taskswitcher=0;
	unsigned char dummy;
	
	//Define data direction and intial conditions for various uC pins used
 	
	TRISC=0b10111110;
	DTR_PIN=LOW;
	TRISB=0b00000001;
	LCD_EN=0;
	LCD_RS=1;
	RTC_CS=1;
	busyLED=LOW;
	
	TRISE=0;
	ADCON1=0x06;
	CMCON=0x07;
	TRISA=0b11110000;
	LATA=0b00000110;
	
	//ENd of define data direction and intial conditions for various uC pins used
	
	gie_off;
	serial_setup();
	lcd_init();
	DelayMs(10);
	lcd_puts("CF Logger");
	
	
	CF_init();
	findRecordsCount(); //find the already written datacount in the CF card
						//This value is written to 

	system_init();  //intial intialisation routine
					//if datalogger is connected to PC serial port and any character is sent by the PC
					//datalogger is put into a SETUP-RETRIEVAl mode
					//where clock can be set, the logger can be reset, or saved data can be retrieved
	CREN=0;

	//Interrupt & timer initialisation
	
	IPEN=0;
	PEIE=1;
	RCIE=1;
	//T08BIT=0;
	//T0CS=0;
	//T0PS2=0;
	//T0PS1=0;
	//T0PS0=1;
	//TMR0IE=1;
	//TMR0ON=1;
	CREN=1;
	DTR_PIN=HIGH;
	gie_on;
	
		
	while(1){
		
		if (newDatReceived) {
			
			gie_off;  		//disable all interrupts during CF wite cycle
			CREN=0;   		//also disable serial port reception
			DTR_PIN=LOW;	//indicate not ready to DCE when hardware flow-control is used 
			busyLED=HIGH;	//Turn on busy LED in hardware
			
			/* 
			The following lines fills cfBuffer[] with date-time data
			This information is exactly 20bytes long
			*/
			cfBuffer[0]=((dateTime.year/10)+'0');cfBuffer[1]=((dateTime.year%10)+'0');
			cfBuffer[2]=('-');
			cfBuffer[3]=((dateTime.month/10)+'0');cfBuffer[4]=((dateTime.month%10)+'0');
			cfBuffer[5]=('-');
			cfBuffer[6]=((dateTime.dateOfTheMonth/10)+'0');cfBuffer[7]=((dateTime.dateOfTheMonth%10)+'0');
			cfBuffer[8]=(' ');
			cfBuffer[9]=(' ');
			cfBuffer[10]=((dateTime.hours/10)+'0');cfBuffer[11]=((dateTime.hours%10)+'0');
			cfBuffer[12]=(':');
			cfBuffer[13]=((dateTime.minutes/10)+'0');cfBuffer[14]=((dateTime.minutes%10)+'0');
			cfBuffer[15]=(':');
			cfBuffer[16]=((dateTime.seconds/10)+'0');cfBuffer[17]=((dateTime.seconds%10)+'0');
			cfBuffer[18]=(' ');
			cfBuffer[19]=(' ');
			for (i=recordLength+1;i<512;i++)cfBuffer[i]=0; //fills the remaining positions of cfBuffer[] with zeros
			
			CF_CurrSector=recordsCount+(RECORDS_COUNT_SECTOR+1);   // records are saved in CF with an offset of RECORDS_COUNT_SECTOR+1 
											//this can safely be set to anything higher than RECORDS_COUNT_SECTOR
			
			CF_StartSectorWrite();			//These two lines save the cfBuffer[] to CF card in the position 'CF_CurrSector'
			CF_WriteBuffIntoSector();
			recordsCount++;					//increment the record count
			CF_CurrSector=RECORDS_COUNT_SECTOR;		//RECORDS_COUNT_SECTOR contains the number of records written to CF
											//Sector 100 is read at the start of the program to find the number of sectors already written
			CF_StartSectorWrite();
			CF_WriteLong(recordsCount);
			CF_WriteSkipBytes(508);			//recordsCount is 32bit (4Byte) long integer so skip 508 Bytes to complete sector write
			newDatReceived=0;
			
			CREN=1;							//enable serial port reception
			DTR_PIN=HIGH;					//tell the DCE I'm not busy now
			gie_on;							//turn on interrupts
			busyLED=LOW;                   //Turn off busy LED
		}
		
		if (byte0(taskswitcher)==0){		//This happens every 256th time while loop runs and (once every several  milliseconds)
			RTCreadDateTime(&dateTime);		//Read date time from RTC
		}
				
		if(taskswitcher==25000){			//every 25000th cycle while loop runs update LCD with datetime and last data received
			taskswitcher=0;					//This happens in the order of 100s of milliseconds
			updateLCD();					
						
		}
		taskswitcher++;
		
		if (OERR){CREN=0;CREN=1;}			//Incase of serial port errors clear them. this can happen if data was sent while CF is being written							
		if (FERR)dummy=RCREG;				//The chances of this happens is very rare in practical situations
											//because after a complete record is sent by DCE we can safely assume it'll not send another
											//record immediately
											//In the worst case we might loose first few bytes of the record sent
									

	}
	
		
}

void interrupt myISR(void){
	
	static unsigned int currentByteNum=20; //the first 20 bytes cotains the date-time data
	unsigned char temp;
	
	
	
	if (RCIF && RCIE){
		
		//serial port reception is handled in an interrupt routine in order to prevent any data loss
		temp=RCREG;	
		CREN=0;			//disable serial port data reception
		DTR_PIN=LOW;	//indicate it to the DCE when hardware flow-control is used
		
		
		if ((temp == 13) || (currentByteNum==511)){ 
		//if a CRLF detected or maximum record length((512-20=492)Bytes) reached 
		//indicate it to main while loop by setting  'newDatReceived'
			cfBuffer[currentByteNum]=0;  
			newDatReceived=1;  //this volatile variable is checked in main while(1) loop
			recordLength=currentByteNum;
			
			currentByteNum=20; //set currentByteNum back to 20 (first 20bytes is datetime data)
			return;
		}
		
		cfBuffer[currentByteNum]=temp; //fills the cfBuffer[] with serial data received by serial port
		currentByteNum++;
		CREN=1;                       //enable reception
		DTR_PIN=HIGH;
	}
}
	

void updateLCD(){
	
	static volatile bit temp; //declared volatile because updateLCD() can be interrupted
	unsigned char i;
	
	if (temp==0) {			//every other time update datetime in LCD
		lcd_goto(10);
		DelayUs(255);
		lcd_PutchDec(dateTime.year);lcd_data('-');
		lcd_PutchDec(dateTime.month);lcd_data('-');
		lcd_PutchDec(dateTime.dateOfTheMonth);lcd_data(' ');
		lcd_PutchDec(dateTime.hours);lcd_data(':');
		lcd_PutchDec(dateTime.minutes);lcd_data(':');
		lcd_PutchDec(dateTime.seconds);
		lcd_data(' ');lcd_data(' ');
		
		lcd_data('R');
		lcd_data('C');
		lcd_data(' ');
		lcd_putchlong(recordsCount);
			
		temp=1;
	}
	else {				//every other time update LCD with first 40bytes of data received
		lcd_cmd(192);
		DelayUs(255);
		for(i=20;i<60;i++)lcd_data(cfBuffer[i]);
		temp=0;
	}
}


void findRecordsCount(){	//find the records already written to CF card

	CF_CurrSector=RECORDS_COUNT_SECTOR;		
	CF_StartSectorRead();
	recordsCount=CF_ReadLong();
	CF_ReadSkipBytes(508);

}

 void system_init(){
	unsigned long k;
	unsigned char dummy;
	putst("Press any key withing 3 seconds to enter setup..\n\n");

	CREN=1; //enable reception
	DelayMs(50);
	k=999999;
	while(!RCIF){
			--k;
	if (k==0) return;
	}
				
	dummy=RCREG;
	putst("Select Function..\n\n"
		"1. Setup Date and Time..\n"
		"2. Intialise Records to zero.."
		"[Current Record Count=");
	putchlong(recordsCount);
	putst("]\n");
	putst("3. Read Records..\n");

	dummy=getch();
	if (dummy=='1') { setDateTime(); return; } 
	if (dummy=='2') {
		recordsCount=0;
		putst("\nRecords Initialised to zero\n");
		return;
	}
	if (dummy=='3') { readRecords(); return;}
	putst("\nWrong Choice. Reset system to run setup again\n");	

	
}

void setDateTime(){
	//set the datetime of DS1288X/DS1788X RTC
	char buf[10];
	putst("Date and Time Setup..\n\n\tEnter year [XX]..");
	USARTGetString(buf, 2);
	dateTime.year=atoi(buf);
	putst("\n\tEnter month [XX]..");USARTGetString(buf, 2);dateTime.month=atoi(buf);
	putst("\n\tEnter date [XX]..");USARTGetString(buf, 2);dateTime.dateOfTheMonth=atoi(buf);
	putst("\n\tEnter day of the week [X]..");USARTGetString(buf, 1);dateTime.dayOfTheWeek=atoi(buf);
	putst("\n\tEnter hour [XX]..");USARTGetString(buf, 2);dateTime.hours=atoi(buf);
	putst("\n\tEnter minutes [XX]..");USARTGetString(buf, 2);dateTime.minutes=atoi(buf);
	dateTime.seconds=0;
	
	RTCintialise(&dateTime,0);

}


unsigned char USARTGetString(char *buffer, unsigned char bufferLen)
{
    //This is a very simple string reading function from serial port
	//can be used when scanf is too heavy or not available
	unsigned char v;
    unsigned char count;

    count = 0;
    do
    {
        while(!RCIF);

        v = RCREG;

        if ( v == '\r' || v == '\n' )
            break;

        count++;
        *buffer++ = v;
        *buffer = '\0';
        if ( bufferLen-- == 0 )
            break;
    } while(1);
    return count;
}

void readRecords(){
//read records stored in CF card and dump them to PC via serial port
//directly connect CF card to PC and use a program like WinHEX for faster data retrieval
//Serial port dump is very slow at even 112k baud

unsigned long rcIndex;
unsigned int first, last;
	unsigned char dummy;
	char buf[10];
	putst("Select Function..\n\n"
		"1. Dump all sectors..\n"
		"2. Select range..[0 - 65535]\n");
	dummy=getch();
	if (dummy=='1') {
	 	for(rcIndex=0;rcIndex<recordsCount;rcIndex++){
			CF_CurrSector=rcIndex+101;
			CF_DumpSector();
		}

		putst("\nDumping Completed\n");
		return;
	} 	
	if (dummy=='2') {
		putst("\nEnter first record No..");
		USARTGetString(buf, 5);
		first=atoi(buf);
		putst("\nEnter last record No..");
		USARTGetString(buf, 5);
		last=atoi(buf);
		for(rcIndex=first;rcIndex<last;rcIndex++){
			CF_CurrSector=rcIndex+101;
			CF_DumpSector();
		}
		putst("\nRead Completed\n");
		return;
	}

	putst("\nWrong Key\n");
}
